#! /usr/bin/env perl
################################################################################
#     The MOLCAS's platform compatibility checker                              #
#     Version 1.0                                                              #
#                                                                              #
#     Victor. P. Vysotskiy, 2014                                               #
#     Copyright: All rights are reserved                                       #
################################################################################
use strict;
use warnings;
use diagnostics;
use Cwd;
#Define reference values
my ($x_ref,$y_ref) =(-0.058230, 0.0);
#Declare variables
my $exe,my $exe_run,my $uexe,my $ntms,my $nexe,my $specs,my $line,my $rc,my $dir;
my @stdout,my @grad,my @mpiruns,my @exedirs,my @menv;

@exedirs=split(/:/,$ENV{"PATH"});
foreach $exe (@exedirs) {
        $exe_run="$exe/mpirun";
        push(@mpiruns,$exe_run) if (((-e $exe_run && -f $exe_run) || -l $exe_run) &&  -x $exe_run);
        $exe_run="$exe/mpiexec";
        push(@mpiruns,$exe_run) if (((-e $exe_run && -f $exe_run) || -l $exe_run) &&  -x $exe_run);
}

$nexe=@mpiruns;
$uexe=0;
if ($nexe>=1) {
   print "Please select proper MPI parallel job launcher:\n";
   $ntms=2;
   foreach(@mpiruns) {
     printf ("[%2d] %s \n",$uexe,$_);
     $uexe++;
   }
   printf ("Your choice [0-%d]: ",$nexe-1);
   READ:
     goto FINISH if($ntms==0);
     $uexe=<>;
     if($uexe!~/^[+-]?\d+$/||$uexe>$nexe-1) {
        print "Wrong answer! Please try one more time\n";
        $ntms--;
        goto READ;
     }
} else {
  print "Sorry, but neitheir 'mpirun' nor 'mpiexec' is found in PATH. Please update your PATH to include MPI library!\n";
  goto FINISH;
}
my $launcher=$mpiruns[$uexe];
print "The MPI parallel job launcher '$launcher' has been choosen for checking\n";

#Read platform
open(FILE,'.platform') or die "Cannot open .platform specification: $!\n";
LINE:
while($line=<FILE>) {
     chomp($line);
     next LINE if ($line=~/^\s*$/||$line=~/#/);
     $specs=$line;
}
close FILE;

#Extract data
$rc=system("tar -xf data.tar");
if ($rc!=0) {
   die "Cannot unpack the 'data.tar' archieve\n";
}
$dir=getcwd;
#Fix molcas.env file
open(FILE,'+<./data/molcas.env') or die "Cannot open data/molcas.env file: $!\n";
$line="CPUS=2\n";
push(@menv,$line);
LINE:
while($line=<FILE>) {
     next if ($line=~/^#/);
     $line="WorkDir=$dir/data/\n" if($line=~/WorkDir/);
     push(@menv,$line);
}
seek(FILE,0,0);
print FILE @menv;
close FILE;
#Copy input and scratch files
dcopy("./data/","./data/tmp_1/");
dcopy("./data/data/","./data/tmp_1/data/");

#Enter working directory
chdir("data") or die "cannot change: $!\n";

#Fix environment
$ENV{"CPUS"}=2;
$ENV{"CurrDir"}="$dir/data";

#Execute binary
chmod 0755, "test.exe";
@stdout=qx#$launcher -np 2 $dir/data/test.exe#;
$rc=$?>>8;
goto FINISH if($rc!=0);

#Analyze results
foreach $line (@stdout) {
           $line=~s/^\s+|\s+$//g;
           @grad=split(/\s+/,$line) if($line=~/H1/);
}
$rc=1 if (($grad[1]!=$x_ref) || ($grad[2]!=$y_ref) || ($grad[2]!=$grad[3]) || "@stdout"!~/2 cores/);

#Inform an end-user
FINISH:
if($rc!=0) {
  print "Unfortunately, the choosen >>> $specs <<< binaries are not suitable for your platform. Please try another one!\n ";
} else {
  print "Congratulations! Your platform is suitable for the choosen >>> $specs <<< binary distribution!\n ";
}
#Clean up
chdir($dir) or die "Cannot change the directory: $!\n";
cleanup("data");

FINISH:
exit;

sub cleanup {
my $dir = shift;
local *DIR;

opendir DIR, $dir or die "Cannot clean up $dir, please do it manually: rm -rf $dir ";
for (readdir DIR) {
     next if /^\.{1,2}$/;
     my $path = "$dir/$_";
     unlink $path if -f $path;
     cleanup($path) if -d $path;
}

closedir DIR;
rmdir $dir or print "Error - $!";
}

sub dcopy {
my ($src,$dst)=@_;
my  $srcf;
local *DIR;
if(-d $src) {
   opendir  DIR, $src or die "Cannot open $src: $!\n";
   mkdir $dst unless (-d $dst);
   for (readdir DIR) {
        next if /^\.{1,2}$/;
        my $srcf="$src/$_";
        my $dstf="$dst/$_";
        next if -d $srcf;
        system("cp $srcf $dstf");
   }
   closedir DIR;
}
}
